diff -urN src_old_old/http/modules/ngx_http_full_status_module.c src/http/modules/ngx_http_full_status_module.c
--- src_old_old/http/modules/ngx_http_full_status_module.c	1970-01-01 03:00:00.000000000 +0300
+++ src/http/modules/ngx_http_full_status_module.c	2009-10-18 19:53:39.000000000 +0400
@@ -0,0 +1,145 @@
+/*
+ * Rebrikov Alexander, Reg.ru
+ */
+
+
+#include <ngx_config.h>
+#include <ngx_core.h>
+#include <ngx_http.h>
+
+
+static char *ngx_http_set_full_status(ngx_conf_t *cf, ngx_command_t *cmd,
+                                 void *conf);
+
+static ngx_command_t  ngx_http_full_status_commands[] = {
+
+    { ngx_string("full_status"),
+      NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_http_set_full_status,
+      0,
+      0,
+      NULL },
+
+      ngx_null_command
+};
+
+
+
+static ngx_http_module_t  ngx_http_full_status_module_ctx = {
+    NULL,                                  /* preconfiguration */
+    NULL,                                  /* postconfiguration */
+
+    NULL,                                  /* create main configuration */
+    NULL,                                  /* init main configuration */
+
+    NULL,                                  /* create server configuration */
+    NULL,                                  /* merge server configuration */
+
+    NULL,                                  /* create location configuration */
+    NULL                                   /* merge location configuration */
+};
+
+
+ngx_module_t  ngx_http_full_status_module = {
+    NGX_MODULE_V1,
+    &ngx_http_full_status_module_ctx,      /* module context */
+    ngx_http_full_status_commands,              /* module directives */
+    NGX_HTTP_MODULE,                       /* module type */
+    NULL,                                  /* init master */
+    NULL,                                  /* init module */
+    NULL,                                  /* init process */
+    NULL,                                  /* init thread */
+    NULL,                                  /* exit thread */
+    NULL,                                  /* exit process */
+    NULL,                                  /* exit master */
+    NGX_MODULE_V1_PADDING
+};
+
+
+static ngx_int_t ngx_http_full_status_handler(ngx_http_request_t *r)
+{
+    size_t             size;
+    ngx_int_t          rc;
+    ngx_buf_t         *b;
+    ngx_chain_t        out;
+    ngx_atomic_int_t   rq, at;
+
+    if (r->method != NGX_HTTP_GET && r->method != NGX_HTTP_HEAD) {
+        return NGX_HTTP_NOT_ALLOWED;
+    }
+
+    rc = ngx_http_discard_request_body(r);
+
+    if (rc != NGX_OK) {
+        return rc;
+    }
+
+    r->headers_out.content_type.len = sizeof("text/plain") - 1;
+    r->headers_out.content_type.data = (u_char *) "text/plain";
+
+    if (r->method == NGX_HTTP_HEAD) {
+        r->headers_out.status = NGX_HTTP_OK;
+
+        rc = ngx_http_send_header(r);
+
+        if (rc == NGX_ERROR || rc > NGX_OK || r->header_only) {
+            return rc;
+        }
+    }
+    
+    size_t server_count = *(ngx_servers_stat->server_stat_count);
+    size = sizeof("vihost requests active_time(msec)\n") + //   
+           2 * 11  * server_count +                                        //    
+           (sizeof("  \n")) * server_count;                           //     
+                                                            
+    size_t i;
+    size_t cl = 128;
+    for (i = 0; i < server_count; i++) {
+        ngx_server_stat_t *stat_server = (ngx_server_stat_t *) (((u_char *)*(ngx_servers_stat->server_stat)) + i * cl);
+        size += stat_server->server_name->len;  //     
+    }
+     
+    b = ngx_create_temp_buf(r->pool, size);
+    if (b == NULL) {
+        return NGX_HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    out.buf = b;
+    out.next = NULL;
+    
+    b->last = ngx_cpymem(b->last, "vihost requests active_time(msec)\n", 
+        sizeof("vihost requests active_time(msec)\n") - 1);
+    for (i = 0; i < server_count; i++) {
+        ngx_server_stat_t * server = (ngx_server_stat_t *) (((u_char *)*(ngx_servers_stat->server_stat)) + i * cl);
+        rq = *server->ngx_stat_requests;
+        at = *server->ngx_stat_time;
+
+        b->last = ngx_cpymem(b->last, server->server_name->data, server->server_name->len - 1);
+        b->last = ngx_sprintf(b->last, " %uA %uA\n",  rq, at);
+    }
+    
+    b->last = ngx_sprintf(b->last, "\0");
+    r->headers_out.status = NGX_HTTP_OK;
+    r->headers_out.content_length_n = b->last - b->pos;
+
+    b->last_buf = 1;
+
+    rc = ngx_http_send_header(r);
+
+    if (rc == NGX_ERROR || rc > NGX_OK || r->header_only) {
+        return rc;
+    }
+
+    return ngx_http_output_filter(r, &out);
+}
+
+
+static char *ngx_http_set_full_status(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
+{
+    ngx_http_core_loc_conf_t  *clcf;
+
+    clcf = ngx_http_conf_get_module_loc_conf(cf, ngx_http_core_module);
+    clcf->handler = ngx_http_full_status_handler;
+
+    return NGX_CONF_OK;
+}
diff -urN src_old_old/http/ngx_http.c src/http/ngx_http.c
--- src_old_old/http/ngx_http.c	2009-05-18 00:13:29.000000000 +0400
+++ src/http/ngx_http.c	2009-10-18 19:09:31.000000000 +0400
@@ -8,6 +8,25 @@
 #include <ngx_core.h>
 #include <ngx_http.h>
 
+#if (NGX_FULL_STATUS)
+
+#define START_STAT_BUF 1024
+#define MULTIPLIER_STAT_BUF 2
+
+ngx_servers_stat_t  ngx_servers_stat0;
+ngx_servers_stat_t *ngx_servers_stat = (ngx_servers_stat_t *) &ngx_servers_stat;
+
+ngx_int_t          http_uri_offset0;
+ngx_int_t          *http_uri_offset = &http_uri_offset0;
+
+ngx_int_t realloc_stat_buf();
+ngx_int_t alloc_stat_buf(size_t n);
+ngx_int_t init_stat_server(ngx_server_stat_t *new_server, ngx_str_t server_name);
+
+static ngx_int_t ngx_http_module_init(ngx_cycle_t *cycle);
+
+#endif
+
 
 static char *ngx_http_block(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
 static ngx_int_t ngx_http_init_phases(ngx_conf_t *cf,
@@ -106,7 +125,11 @@
     ngx_http_commands,                     /* module directives */
     NGX_CORE_MODULE,                       /* module type */
     NULL,                                  /* init master */
+#if (NGX_FULL_STATUS)    
+    ngx_http_module_init,                  /* init module */
+#else
     NULL,                                  /* init module */
+#endif
     NULL,                                  /* init process */
     NULL,                                  /* init thread */
     NULL,                                  /* exit thread */
@@ -128,7 +151,7 @@
     ngx_http_core_loc_conf_t    *clcf;
     ngx_http_core_srv_conf_t   **cscfp;
     ngx_http_core_main_conf_t   *cmcf;
-
+    
     /* the main http context */
 
     ctx = ngx_pcalloc(cf->pool, sizeof(ngx_http_conf_ctx_t));
@@ -2006,3 +2029,159 @@
 
     return NGX_OK;
 }
+
+#if (NGX_FULL_STATUS)
+static ngx_int_t ngx_http_module_init(ngx_cycle_t *cycle) {
+    u_char              *shared;
+    size_t               size, cl;
+    ngx_shm_t            shm;
+    
+    size_t server_stat_size = sizeof( ngx_servers_stat );
+    cl = 128;
+    int div = server_stat_size / cl;
+    int mod = server_stat_size % cl;
+    server_stat_size = (div + mod ? 1 : 0) * cl;
+    size = server_stat_size;
+    size += 5 * cl;
+
+    shm.size = size;
+    shm.name.len = sizeof("nginx_shared_zone");
+    shm.name.data = (u_char *) "nginx_shared_zone";
+    shm.log = cycle->log;
+
+    if (ngx_shm_alloc(&shm) != NGX_OK) {
+        return NGX_ERROR;
+    }
+
+    shared = shm.addr;
+
+    ngx_servers_stat = (ngx_servers_stat_t *) (shared + 0 * cl);
+    ngx_servers_stat->server_stat       = (ngx_server_stat_t **) (shared + 1 * cl);
+    ngx_servers_stat->server_stat_count = (size_t *)             (shared + 2 * cl);
+    ngx_servers_stat->alloced           = (size_t *)             (shared + 3 * cl);
+    ngx_servers_stat->log               = (ngx_log_t **)         (shared + 4 * cl);
+    ngx_servers_stat->lock_data         = cycle->lock_file.data;
+    *(ngx_servers_stat->server_stat_count) = 0;
+    *(ngx_servers_stat->log) = shm.log;
+
+    if ( alloc_stat_buf(START_STAT_BUF) != NGX_OK )
+        return NGX_ERROR;
+
+    return NGX_OK;
+}
+
+ngx_server_stat_t *find_stat(ngx_str_t server_name) {
+    size_t i;
+    size_t cl = 128;
+
+    /*       cl.    ,  array[i]   */
+    for (i = 0; i < *ngx_servers_stat->server_stat_count; i++) {
+        ngx_server_stat_t *stat_server = (ngx_server_stat_t *) (((u_char *)*(ngx_servers_stat->server_stat)) + i * cl);
+                                   
+        if (ngx_strncmp(stat_server->server_name->data, server_name.data, server_name.len) == 0)
+            return stat_server;
+    }
+    
+    (*ngx_servers_stat->server_stat_count)++;
+    if (*ngx_servers_stat->server_stat_count > *ngx_servers_stat->alloced)
+        realloc_stat_buf();
+    
+    ngx_server_stat_t *new_server = (ngx_server_stat_t *) (((u_char *) *(ngx_servers_stat->server_stat)) + (*ngx_servers_stat->server_stat_count - 1) * cl);
+    
+    if ( init_stat_server(new_server, server_name) == NGX_OK )
+        return new_server;
+    else
+        return NULL;
+}
+
+ngx_int_t realloc_stat_buf() {
+    size_t cl;
+        
+    cl = 128;
+    ngx_server_stat_t *old_buf = *(ngx_servers_stat->server_stat);
+    ngx_server_stat_t *new_buf;
+    size_t old_alloced = *ngx_servers_stat->alloced;
+    
+    ngx_log_error(NGX_LOG_ERR, *(ngx_servers_stat->log), 0, "test1");    
+    if ( alloc_stat_buf(*ngx_servers_stat->alloced * MULTIPLIER_STAT_BUF) != NGX_OK )
+        return NGX_ERROR;
+    
+    new_buf = *(ngx_servers_stat->server_stat);
+    ngx_log_error(NGX_LOG_ERR, *(ngx_servers_stat->log), 0, "test2");    
+    if (!memcpy(new_buf, old_buf, old_alloced * cl)) {
+            return NGX_ERROR;
+    }
+    
+    ngx_log_error(NGX_LOG_ERR, *(ngx_servers_stat->log), 0, "test3");
+    /* ,   */
+    return NGX_OK;
+}
+
+ngx_int_t alloc_stat_buf(size_t n) {
+    size_t       size, cl;
+    ngx_shm_t    shm;
+    
+    cl = 128;
+    size = n * cl;
+    shm.size = size;
+    shm.log = *(ngx_servers_stat->log);
+    shm.name.len = sizeof("ngx_servers_stat_t");
+    shm.name.data = (u_char *) "ngx_servers_stat_t";
+
+    if (ngx_shm_alloc(&shm) != NGX_OK) {
+        return NGX_ERROR;
+    }
+       
+    *(ngx_servers_stat->server_stat) = (ngx_server_stat_t *) shm.addr;
+    *(ngx_servers_stat->alloced) = n;
+    return NGX_OK;
+}
+
+ngx_int_t init_stat_server(ngx_server_stat_t *new_server, ngx_str_t server_name) {
+    u_char              *shared;
+    size_t               size, cl;
+    ngx_shm_t            shm;
+            
+
+    cl = 128;
+    size = 0;
+    
+    /*                                     */
+    /*     cl.      */
+    /*    div * cl.   ,    */
+    /*     cl                                       */
+    /*  +1    \0  ngx_cpystrn                 */
+    int div = (server_name.len + 1) / cl;
+    int mod = (server_name.len + 1) % cl;
+    int str_size = (div + mod ? 1 : 0) * cl;
+    size += str_size;
+    
+    /*      ngx_server_stat_t */
+    size += 4 * cl;
+    
+    shm.size = size;
+    /*        \0 */
+    shm.name.len = server_name.len;
+    shm.name.data = (u_char *) malloc(shm.name.len + 1);
+    /* ngx_cpystrn   \0,        */
+    ngx_cpystrn(shm.name.data, server_name.data, server_name.len);
+    shm.log = *ngx_servers_stat->log;
+
+    if (ngx_shm_alloc(&shm) != NGX_OK) {
+        return NGX_ERROR;
+    }
+
+    shared = shm.addr;
+
+    new_server->server_name       = (ngx_str_t *)    shared + cl * 0;
+    new_server->ngx_stat_time     = (ngx_atomic_t *) shared + cl * 1;
+    new_server->ngx_stat_requests = (ngx_atomic_t *) shared + cl * 2;
+    
+    new_server->server_name->len  = server_name.len + 1;
+    new_server->server_name->data = (u_char *) (ngx_atomic_t *) shared + cl * 3;
+
+    /*    */
+    ngx_cpystrn(new_server->server_name->data, server_name.data, server_name.len + 1);
+    return NGX_OK;
+}
+#endif
diff -urN src_old_old/http/ngx_http.h src/http/ngx_http.h
--- src_old_old/http/ngx_http.h	2009-05-08 18:52:50.000000000 +0400
+++ src/http/ngx_http.h	2009-10-18 18:02:12.000000000 +0400
@@ -11,6 +11,24 @@
 #include <ngx_config.h>
 #include <ngx_core.h>
 
+#if (NGX_FULL_STATUS)
+typedef struct {
+    ngx_str_t      *server_name;
+    ngx_atomic_t   *ngx_stat_time;    
+    ngx_atomic_t   *ngx_stat_requests;       
+} ngx_server_stat_t;
+
+typedef struct {
+    ngx_server_stat_t **server_stat;
+    size_t             *server_stat_count;
+    size_t             *alloced;
+    ngx_log_t          **log;
+    u_char              *lock_data;   
+} ngx_servers_stat_t;
+
+extern ngx_servers_stat_t *ngx_servers_stat;
+ngx_server_stat_t *find_stat(ngx_str_t server_name);
+#endif
 
 typedef struct ngx_http_request_s     ngx_http_request_t;
 typedef struct ngx_http_upstream_s    ngx_http_upstream_t;
diff -urN src_old_old/http/ngx_http_request.c src/http/ngx_http_request.c
--- src_old_old/http/ngx_http_request.c	2009-09-07 15:11:24.000000000 +0400
+++ src/http/ngx_http_request.c	2009-10-17 22:51:08.000000000 +0400
@@ -251,7 +251,7 @@
 #endif
 
     c = rev->data;
-
+    
     if (rev->timedout) {
         ngx_log_error(NGX_LOG_INFO, c->log, NGX_ETIMEDOUT, "client timed out");
 
@@ -2795,6 +2795,29 @@
     }
 
 #endif
+#if (NGX_FULL_STATUS)
+    if (r->headers_in.host) {
+        ngx_str_t vihost_name = r->headers_in.host->value;
+        ngx_server_stat_t *cserver = find_stat(vihost_name);    
+        if (cserver) {
+            ngx_time_t      *tp;
+            ngx_msec_int_t   ms;
+            
+            tp = ngx_timeofday();
+            ms = (ngx_msec_int_t)
+                ((tp->sec - r->start_sec) * 1000 + (tp->msec - r->start_msec));
+            
+            ngx_log_error(NGX_LOG_ERR, *(ngx_servers_stat->log), 0,
+                   "ms: %d", ms);
+                    
+            ms = (ms >= 0) ? ms : 0;
+            ngx_atomic_fetch_add(cserver->ngx_stat_time, ms);
+            ngx_atomic_fetch_add(cserver->ngx_stat_requests, 1);
+        }
+    }
+
+#endif
+
 
     if (error && r->headers_out.status == 0) {
         r->headers_out.status = error;
@@ -2874,7 +2897,6 @@
 #if (NGX_STAT_STUB)
     (void) ngx_atomic_fetch_add(ngx_stat_active, -1);
 #endif
-
     c->destroyed = 1;
 
     pool = c->pool;
diff -urN src_old_old/http/ngx_http_request.h src/http/ngx_http_request.h
--- src_old_old/http/ngx_http_request.h	2009-05-22 15:05:26.000000000 +0400
+++ src/http/ngx_http_request.h	2009-10-17 21:54:04.000000000 +0400
@@ -501,6 +501,10 @@
     unsigned                          stat_reading:1;
     unsigned                          stat_writing:1;
 #endif
+#if (NGX_FULL_STATUS)
+    unsigned                          full_stat_reading:1;
+    unsigned                          full_stat_writing:1;
+#endif
 
     unsigned                          subrequests:8;
 